package de.jottyfan.bico.modules.item;

import static de.jottyfan.bico.db.Tables.T_LESSON;
import static de.jottyfan.bico.db.Tables.T_LESSON_SUBJECT;
import static de.jottyfan.bico.db.Tables.T_PERSON;
import static de.jottyfan.bico.db.Tables.T_SLOT;
import static de.jottyfan.bico.db.Tables.T_SOURCE;
import static de.jottyfan.bico.db.Tables.T_SUBJECT;

import java.time.LocalDate;
import java.util.Iterator;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.jooq.Record17;
import org.jooq.SelectConditionStep;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.bico.modules.item.model.ItemBean;
import de.jottyfan.bico.modules.item.model.LessonBean;
import de.jottyfan.bico.modules.item.model.PersonBean;
import de.jottyfan.bico.modules.item.model.SourceBean;
import de.jottyfan.bico.modules.item.model.SubjectBean;

/**
 *
 * @author jotty
 *
 */
@Repository
public class ItemRepository {
	private static final Logger LOGGER = LogManager.getLogger(ItemRepository.class);

	@Autowired
	private DSLContext jooq;

	/**
	 * get the item of slot
	 *
	 * @param slot the ID of the slot
	 * @return the bean
	 */
	public ItemBean getItem(Integer slot) {
		SelectConditionStep<Record17<Integer, LocalDate, String, String, Integer, String, String, String, Integer, String, Integer, String, String, String, String, String, String>> sql = jooq
		// @formatter:off
			.select(T_SLOT.PK_SLOT,
					    T_SLOT.SLOT_DAY,
					    T_SLOT.NOTE,
					    T_LESSON.NOTES,
					    T_PERSON.PK_PERSON,
					    T_PERSON.FORENAME,
					    T_PERSON.SURNAME,
					    T_PERSON.ABBREVIATION,
					    T_SOURCE.PK_SOURCE,
					    T_SOURCE.NAME,
					    T_SUBJECT.PK_SUBJECT,
					    T_SUBJECT.THEME,
					    T_SUBJECT.SUBTHEME,
					    T_SUBJECT.WORKSHEETS,
					    T_SUBJECT.BIBLEVERSE,
					    T_SUBJECT.BOOK_PAGES,
					    T_SOURCE.NAME)
			.from(T_SLOT)
			.leftJoin(T_LESSON).on(T_LESSON.FK_SLOT.eq(T_SLOT.PK_SLOT))
			.leftJoin(T_PERSON).on(T_PERSON.PK_PERSON.eq(T_LESSON.FK_PERSON))
			.leftJoin(T_LESSON_SUBJECT).on(T_LESSON_SUBJECT.FK_LESSON.eq(T_LESSON.PK_LESSON))
			.leftJoin(T_SUBJECT).on(T_SUBJECT.PK_SUBJECT.eq(T_LESSON_SUBJECT.FK_SUBJECT))
			.leftJoin(T_SOURCE).on(T_SOURCE.PK_SOURCE.eq(T_SUBJECT.FK_SOURCE))
			.where(T_SLOT.PK_SLOT.eq(slot));
		// @formatter:on
		LOGGER.trace(sql);
		Iterator<Record17<Integer, LocalDate, String, String, Integer, String, String, String, Integer, String, Integer, String, String, String, String, String, String>> i = sql
				.fetch().iterator();
		ItemBean bean = new ItemBean();
		while (i.hasNext()) {
			Record17<Integer, LocalDate, String, String, Integer, String, String, String, Integer, String, Integer, String, String, String, String, String, String> r = i
					.next();
			bean.setPkSlot(r.get(T_SLOT.PK_SLOT));
			bean.setSlotDay(r.get(T_SLOT.SLOT_DAY));
			bean.setSlotNote(r.get(T_SLOT.NOTE));
			bean.getLessons()
					.add(LessonBean.of(
							r.get(T_LESSON.NOTES), new PersonBean(r.get(T_PERSON.PK_PERSON), r.get(T_PERSON.FORENAME),
									r.get(T_PERSON.SURNAME), r.get(T_PERSON.ABBREVIATION)),
							new SubjectBean(new SourceBean(r.get(T_SOURCE.PK_SOURCE), r.get(T_SOURCE.NAME)), r.get(T_SUBJECT.PK_SUBJECT), r.get(T_SUBJECT.THEME),
									r.get(T_SUBJECT.SUBTHEME), r.get(T_SUBJECT.WORKSHEETS), r.get(T_SUBJECT.BIBLEVERSE),
									r.get(T_SUBJECT.BOOK_PAGES))));
		}
		return bean;
	}
}
